@@ -28,9 +28,11 @@
 
 G_BEGIN_DECLS
 
-GUPnPDIDLLiteObject *
+G_GNUC_INTERNAL GUPnPDIDLLiteObject *
 gupnp_didl_lite_object_new_from_xml     (xmlNode     *xml_node,
-                                         GUPnPXMLDoc *xml_doc);
+                                         GUPnPXMLDoc *xml_doc,
+                                         xmlNs       *upnp_ns,
+                                         xmlNs       *dc_ns);
 
 G_END_DECLS
 
