@@ -32,7 +32,6 @@
 #include "gupnp-didl-lite-object.h"
 #include "gupnp-didl-lite-object-private.h"
 #include "gupnp-didl-lite-descriptor-private.h"
-#include "gupnp-dlna-private.h"
 
 G_DEFINE_TYPE (GUPnPDIDLLiteWriter,
                gupnp_didl_lite_writer,
@@ -135,6 +134,8 @@ is_standard_prop (const char *name,
                 strcmp (name, "title") == 0) ||
                (g_strcmp0 (namespace, "upnp") == 0 &&
                 strcmp (name, "class") == 0) ||
+               (g_strcmp0 (parent_name, "albumArtURI") == 0 &&
+                strcmp (name, "profileID") == 0) ||
                (g_strcmp0 (parent_name, "res") == 0 &&
                 strcmp (name, "protocolInfo") == 0);
 }
@@ -184,7 +185,9 @@ filter_node (xmlNode             *node,
                 if (child->ns != NULL)
                         ns = (const char *) child->ns->prefix;
 
-                if (!is_standard_prop ((const char *) child->name, ns, NULL) &&
+                if (!is_standard_prop ((const char *) child->name,
+                                       ns,
+                                       (const char *)  node->name) &&
                     is_node_forbidden (child, allowed, ns))
                         forbidden = g_list_append (forbidden, child);
         }
@@ -389,7 +392,7 @@ gupnp_didl_lite_writer_class_init (GUPnPDIDLLiteWriterClass *klass)
 }
 
 /**
- * gupnp_didl_lite_writer_new
+ * gupnp_didl_lite_writer_new:
  * @language: The language the DIDL-Lite fragment is in, or NULL
  *
  * Return value: A new #GUPnPDIDLLiteWriter object.
@@ -403,7 +406,7 @@ gupnp_didl_lite_writer_new (const char *language)
 }
 
 /**
- * gupnp_didl_lite_writer_add_item
+ * gupnp_didl_lite_writer_add_item:
  * @writer: A #GUPnPDIDLLiteWriter
  *
  * Creates a new item, attaches it to @writer and returns it.
@@ -424,12 +427,14 @@ gupnp_didl_lite_writer_add_item (GUPnPDIDLLiteWriter *writer)
                                 NULL);
 
         object = gupnp_didl_lite_object_new_from_xml (item_node,
-                                                      writer->priv->xml_doc);
+                                                      writer->priv->xml_doc,
+                                                      writer->priv->upnp_ns,
+                                                      writer->priv->dc_ns);
         return GUPNP_DIDL_LITE_ITEM (object);
 }
 
 /**
- * gupnp_didl_lite_writer_add_container
+ * gupnp_didl_lite_writer_add_container:
  * @writer: A #GUPnPDIDLLiteWriter
  *
  * Creates a new container, attaches it to @writer and returns it.
@@ -450,12 +455,14 @@ gupnp_didl_lite_writer_add_container (GUPnPDIDLLiteWriter *writer)
                                       NULL);
 
         object = gupnp_didl_lite_object_new_from_xml (container_node,
-                                                      writer->priv->xml_doc);
+                                                      writer->priv->xml_doc,
+                                                      writer->priv->upnp_ns,
+                                                      writer->priv->dc_ns);
         return GUPNP_DIDL_LITE_CONTAINER (object);
 }
 
 /**
- * gupnp_didl_lite_writer_add_descriptor
+ * gupnp_didl_lite_writer_add_descriptor:
  * @writer: A #GUPnPDIDLLiteWriter
  *
  * Creates a new descriptor, attaches it to @object and returns it.
@@ -479,7 +486,7 @@ gupnp_didl_lite_writer_add_descriptor (GUPnPDIDLLiteWriter *writer)
 }
 
 /**
- * gupnp_didl_lite_writer_get_string
+ * gupnp_didl_lite_writer_get_string:
  * @writer: A #GUPnPDIDLLiteWriter
  *
  * Creates a string representation of the DIDL-Lite XML document.
@@ -508,7 +515,7 @@ gupnp_didl_lite_writer_get_string (GUPnPDIDLLiteWriter *writer)
 }
 
 /**
- * gupnp_didl_lite_writer_get_xml_node
+ * gupnp_didl_lite_writer_get_xml_node:
  * @writer: The #GUPnPDIDLLiteWriter
  *
  * Get the pointer to root node in XML document.
@@ -524,7 +531,7 @@ gupnp_didl_lite_writer_get_xml_node (GUPnPDIDLLiteWriter *writer)
 }
 
 /**
- * gupnp_didl_lite_writer_get_language
+ * gupnp_didl_lite_writer_get_language:
  * @writer: #GUPnPDIDLLiteWriter
  *
  * Get the language the DIDL-Lite fragment is in.
@@ -540,7 +547,7 @@ gupnp_didl_lite_writer_get_language (GUPnPDIDLLiteWriter *writer)
 }
 
 /**
- * gupnp_didl_lite_writer_filter
+ * gupnp_didl_lite_writer_filter:
  * @writer: A #GUPnPDIDLLiteWriter
  * @filter: A filter string
  *
