@@ -36,6 +36,56 @@
 #include "gupnp-search-criteria-parser.h"
 #include "gupnp-av-marshal.h"
 
+/* GType for GUPNPSearchCriteriaOp */
+GType
+gupnp_search_criteria_op_get_type (void)
+{
+        static GType type = 0;
+
+        if (type == 0) {
+                static const GEnumValue values[] = {
+                        { GUPNP_SEARCH_CRITERIA_OP_EQ,
+                          "GUPNP_SEARCH_CRITERIA_OP_EQ",
+                          "none" },
+                        { GUPNP_SEARCH_CRITERIA_OP_NEQ,
+                          "GUPNP_SEARCH_CRITERIA_OP_NEQ",
+                          "none" },
+                        { GUPNP_SEARCH_CRITERIA_OP_LESS,
+                          "GUPNP_SEARCH_CRITERIA_OP_LESS",
+                          "none" },
+                        { GUPNP_SEARCH_CRITERIA_OP_LEQ,
+                          "GUPNP_SEARCH_CRITERIA_OP_LEQ",
+                          "none" },
+                        { GUPNP_SEARCH_CRITERIA_OP_GREATER,
+                          "GUPNP_SEARCH_CRITERIA_OP_GREATER",
+                          "none" },
+                        { GUPNP_SEARCH_CRITERIA_OP_GEQ,
+                          "GUPNP_SEARCH_CRITERIA_OP_GEQ",
+                          "none" },
+                        { GUPNP_SEARCH_CRITERIA_OP_CONTAINS,
+                          "GUPNP_SEARCH_CRITERIA_OP_CONTAINS",
+                          "none" },
+                        { GUPNP_SEARCH_CRITERIA_OP_DOES_NOT_CONTAIN,
+                          "GUPNP_SEARCH_CRITERIA_OP_DOES_NOT_CONTAIN",
+                          "none" },
+                        { GUPNP_SEARCH_CRITERIA_OP_DERIVED_FROM,
+                          "GUPNP_SEARCH_CRITERIA_OP_DERIVED_FROM",
+                          "none" },
+                        { GUPNP_SEARCH_CRITERIA_OP_EXISTS,
+                          "GUPNP_SEARCH_CRITERIA_OP_EXISTS",
+                          "none" },
+                        { 0, NULL, NULL }
+                };
+
+                type = g_enum_register_static
+                                (g_intern_static_string (
+                                 "GUPnPSearchCriteriaOp"),
+                                 values);
+        }
+
+        return type;
+}
+
 /* GUPnPSearchCriteriaParserError */
 GQuark
 gupnp_search_criteria_parser_error_quark (void)
@@ -178,12 +228,11 @@ gupnp_search_criteria_parser_class_init
         object_class->finalize = gupnp_search_criteria_parser_finalize;
 
         /**
-         * GUPnPSearchCriteriaParser::begin_parens
-         * @parser: The #GUPnPGUPnPSearchCriteriaParser that received the
-         * signal
+         * GUPnPSearchCriteriaParser::begin-parens
+         * @parser: The #GUPnPSearchCriteriaParser that received the signal
          *
-         * The ::begin_parens signal is emitted to mark the beginning
-         * of a parenthetical expression.
+         * The ::begin_parens signal is emitted to mark the beginning of a
+         * parenthetical expression.
          **/
         signals[BEGIN_PARENS] =
                 g_signal_new ("begin-parens",
@@ -198,12 +247,11 @@ gupnp_search_criteria_parser_class_init
                               0);
 
         /**
-         * GUPnPSearchCriteriaParser::end_parens
-         * @parser: The #GUPnPGUPnPSearchCriteriaParser that received the
-         * signal
+         * GUPnPSearchCriteriaParser::end-parens
+         * @parser: The #GUPnPSearchCriteriaParser that received the signal
          *
-         * The ::end_parens signal is emitted to mark the end
-         * of a parenthetical expression.
+         * The ::end_parens signal is emitted to mark the end of a parenthetical
+         * expression.
          **/
         signals[END_PARENS] =
                 g_signal_new ("end-parens",
@@ -219,11 +267,10 @@ gupnp_search_criteria_parser_class_init
 
         /**
          * GUPnPSearchCriteriaParser::conjunction
-         * @parser: The #GUPnPGUPnPSearchCriteriaParser that received the
-         * signal
+         * @parser: The #GUPnPSearchCriteriaParser that received the signal
          *
          * The ::conjuction signal is emitted whenever a conjuction marker
-         * (and) is parsed.
+         * &lpar;and&rpar; is parsed.
          **/
         signals[CONJUNCTION] =
                 g_signal_new ("conjunction",
@@ -239,11 +286,10 @@ gupnp_search_criteria_parser_class_init
 
         /**
          * GUPnPSearchCriteriaParser::disjunction
-         * @parser: The #GUPnPGUPnPSearchCriteriaParser that received the
-         * signal
+         * @parser: The #GUPnPSearchCriteriaParser that received the signal
          *
          * The ::disjuction signal is emitted whenever a disjuction marker
-         * (or) is parsed.
+         * &lpar;or&rpar is parsed.
          **/
         signals[DISJUNCTION] =
                 g_signal_new ("disjunction",
@@ -259,11 +305,14 @@ gupnp_search_criteria_parser_class_init
 
         /**
          * GUPnPSearchCriteriaParser::expression
-         * @parser: The #GUPnPGUPnPSearchCriteriaParser that received the
-         * signal
+         * @parser: The #GUPnPSearchCriteriaParser that received the signal
+         * @property: The property
+         * @op: The operator as #GUPnPSearchCriteriaOp
+         * @value: The value as string
+         * @error: Place-holder for any possible errors from handler
          *
-         * The ::expression signal is emitted whenever an expression
-         * (or) is parsed. Set @error and return FALSE if an error occurred.
+         * The ::expression signal is emitted whenever an expression is parsed.
+         * Set @error and return %FALSE if an error occurred.
          **/
         signals[EXPRESSION] =
                 g_signal_new ("expression",
@@ -277,7 +326,7 @@ gupnp_search_criteria_parser_class_init
                               G_TYPE_BOOLEAN,
                               4,
                               G_TYPE_STRING,
-                              G_TYPE_UINT,
+                              GUPNP_TYPE_SEARCH_CRITERIA_OP,
                               G_TYPE_STRING,
                               G_TYPE_POINTER);
 
@@ -395,6 +444,50 @@ scan_rel_exp (GUPnPSearchCriteriaParser *parser,
         return ret;
 }
 
+static gboolean
+scan_search_exp (GUPnPSearchCriteriaParser *parser,
+                 GError                   **error);
+
+/* Scan a Logical operator and the part after that */
+static gboolean
+scan_logical_op (GUPnPSearchCriteriaParser *parser,
+                 GError                   **error)
+{
+        gboolean ret;
+        guint token;
+
+        token = g_scanner_peek_next_token (parser->priv->scanner);
+
+        switch (token) {
+                case SYMBOL_AND:
+                        g_scanner_get_next_token (parser->priv->scanner);
+
+                        g_signal_emit (parser, signals[CONJUNCTION], 0);
+
+                        ret = scan_search_exp (parser, error);
+
+                        break;
+
+                case SYMBOL_OR:
+                        g_scanner_get_next_token (parser->priv->scanner);
+
+                        g_signal_emit (parser, signals[DISJUNCTION], 0);
+
+                        ret = scan_search_exp (parser, error);
+
+                        break;
+
+                default:
+
+                        ret = TRUE;
+
+                        break;
+
+        }
+
+        return ret;
+}
+
 /* Scan a searchExp portion of a search criteria string */
 static gboolean
 scan_search_exp (GUPnPSearchCriteriaParser *parser,
@@ -431,6 +524,8 @@ scan_search_exp (GUPnPSearchCriteriaParser *parser,
 
                 g_signal_emit (parser, signals[END_PARENS], 0);
 
+                ret = scan_logical_op (parser, error);
+
                 break;
 
         case G_TOKEN_IDENTIFIER:
@@ -438,30 +533,7 @@ scan_search_exp (GUPnPSearchCriteriaParser *parser,
                 if (ret == FALSE)
                         break;
 
-                token = g_scanner_peek_next_token (parser->priv->scanner);
-
-                switch (token) {
-                case SYMBOL_AND:
-                        g_scanner_get_next_token (parser->priv->scanner);
-
-                        g_signal_emit (parser, signals[CONJUNCTION], 0);
-
-                        ret = scan_search_exp (parser, error);
-
-                        break;
-
-                case SYMBOL_OR:
-                        g_scanner_get_next_token (parser->priv->scanner);
-
-                        g_signal_emit (parser, signals[DISJUNCTION], 0);
-
-                        ret = scan_search_exp (parser, error);
-
-                        break;
-
-                default:
-                        break;
-                }
+                ret = scan_logical_op (parser, error);
 
                 break;
 
